#Author-Brian Ekins (www.EkinsSolutions.com)
#Description-Create a construction point at the specified X,Y,Z coordinates.

import adsk.core, adsk.fusion, adsk.cam, traceback
handlers = []
_isParametric = False
_app = adsk.core.Application.get()
_ui = _app.userInterface
_design = adsk.fusion.Design.cast(None)
_pointsData = []  # A list of lists containing the coordinate (Point3D) and Name of each point.

# Command inputs.
_baseFeatureList = adsk.core.DropDownCommandInput.cast(None)
_nameInput = adsk.core.StringValueCommandInput.cast(None)
_xValInput = adsk.core.ValueCommandInput.cast(None)
_yValInput = adsk.core.ValueCommandInput.cast(None)
_zValInput = adsk.core.ValueCommandInput.cast(None)
_createInput = adsk.core.BoolValueCommandInput.cast(None)


# Event handler for the commandCreated event.
class CommandCreatedEventHandler(adsk.core.CommandCreatedEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        try:
            global _pointsData, _baseFeatureList, _nameInput, _xValInput, _yValInput, _zValInput, _createInput
            _pointsData = []
            
            eventArgs = adsk.core.CommandCreatedEventArgs.cast(args)
            cmd = eventArgs.command
    
            # Connect to the execute event.
            onExecute = CommandExecuteHandler()
            cmd.execute.add(onExecute)
            handlers.append(onExecute)

            # Connect to the inputChanged event.
            onInputChanged = CommandInputChangedHandler()
            cmd.inputChanged.add(onInputChanged)
            handlers.append(onInputChanged)
            
            # Connect ot the preview event.       
            onExecutePreview = ExecutePreviewHandler()
            cmd.executePreview.add(onExecutePreview)
            handlers.append(onExecutePreview)        
            
            # Check to see if this is a parametric or direct-edit model.
            global _design, _isParametric
            _design = adsk.fusion.Design.cast(_app.activeProduct)
            if _design:
                inputs = cmd.commandInputs

                if _design.designType == adsk.fusion.DesignTypes.ParametricDesignType:
                    _isParametric = True

                    # Create a list of base features.
                    _baseFeatureList = inputs.addDropDownCommandInput('baseFeatureList', 'Base feature', adsk.core.DropDownStyles.TextListDropDownStyle) 
                    _baseFeatureList.listItems.add('Create new Base Feature', True, '')                
    
                    # Get the name of the last used base feature to use as the default.
                    baseFeatureNameAttrib = _design.attributes.itemByName('ekinsPointAtCoord', 'BaseFeatureName')
                    if baseFeatureNameAttrib:
                        baseFeatureName = baseFeatureNameAttrib.value
    
                    for baseFeature in _design.rootComponent.features.baseFeatures:
                        if baseFeatureNameAttrib:
                            if baseFeatureName == baseFeature.name:
                                _baseFeatureList.listItems.item(0).isSelected = False
                                _baseFeatureList.listItems.add(baseFeature.name, True, '')                                
                        else:
                            _baseFeatureList.listItems.add(baseFeature.name, False, '')
                else:
                    _isParametric = False
                    
                # Add string input to get the name.
                _nameInput = inputs.addStringValueInput('pointName', 'Name', 'XYZ Point')            
                
                # Add value inputs to get the X, Y, and Z values.
                _xValInput = inputs.addValueInput('xValInput', 'X Position', _design.unitsManager.defaultLengthUnits, adsk.core.ValueInput.createByReal(0))
                _yValInput = inputs.addValueInput('yValInput', 'Y Position', _design.unitsManager.defaultLengthUnits, adsk.core.ValueInput.createByReal(0))
                _zValInput = inputs.addValueInput('zValInput', 'Z Position', _design.unitsManager.defaultLengthUnits, adsk.core.ValueInput.createByReal(0))
                
                # Add the button to create a point at the current location.
                _createInput = inputs.addBoolValueInput('create', 'Create', False, 'Resources/Create')            
            else:
                _ui.messageBox('You must be in a modeling related workspace.')
                return False
        except:
            _ui.messageBox('Unexpected failure:\n{}'.format(traceback.format_exc()))


# Event handler for the executePreview event.
class ExecutePreviewHandler(adsk.core.CommandEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        try:
            eventArgs = adsk.core.CommandEventArgs.cast(args)
            
            # Get the current point data.
            # Get the values from the command inputs.
            xVal = _xValInput.value       
            yVal = _yValInput.value
            zVal = _zValInput.value           
            newPointCoord = adsk.core.Point3D.create(xVal, yVal, zVal)
            
            newPointName = _nameInput.value
    
            DrawPoints(newPointCoord, newPointName)
        except:
            _ui.messageBox('Unexpected failure:\n{}'.format(traceback.format_exc()))
        

# Event handler for the inputChanged event.
class CommandInputChangedHandler(adsk.core.InputChangedEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        try:
            eventArgs = adsk.core.InputChangedEventArgs.cast(args)
            
            if eventArgs.input.id == 'create':
                # Get the values from the command inputs.
                xVal = _xValInput.value
                yVal = _yValInput.value
                zVal = _zValInput.value
                newPointName = _nameInput.value
                
                _pointsData.append([adsk.core.Point3D.create(xVal, yVal, zVal), newPointName])
        except:
            _ui.messageBox('Unexpected failure:\n{}'.format(traceback.format_exc()))


# Event handler for the execute event.
class CommandExecuteHandler(adsk.core.CommandEventHandler):
    def __init__(self):
        super().__init__()
    def notify(self, args):
        try:
            eventArgs = adsk.core.CommandEventArgs.cast(args)           
            
            # Get the values from the command inputs.
            xVal = _xValInput.value
            yVal = _yValInput.value
            zVal = _zValInput.value
            currentPoint = adsk.core.Point3D.create(xVal, yVal, zVal)
            
            # Check to see if the current coordinate already exists in the list.
            found = False
            for pointData in _pointsData:
                if currentPoint.isEqualTo(pointData[0]):
                    found = True
                    break
                
            if not found:
                currentName = _nameInput.value
                _pointsData.append([currentPoint, currentName])
            
            CreatePoints(eventArgs.command.commandInputs)
        except:
            _ui.messageBox('Unexpected failure:\n{}'.format(traceback.format_exc()))


def DrawPoints(newPointCoord, newPointName):
    try:
        root = _design.rootComponent
        
        graphicsGroup = root.customGraphicsGroups.add()
        
        newPointCoords = adsk.fusion.CustomGraphicsCoordinates.create([newPointCoord.x, newPointCoord.y, newPointCoord.z])
        pntSet = graphicsGroup.addPointSet(newPointCoords, [], adsk.fusion.CustomGraphicsPointTypes.UserDefinedCustomGraphicsPointType, 'Resources/NewPoint.png')
        pntSet.depthPriority = 0
        
        if len(_pointsData) > 0:
            pointCoords = []
            for pointData in _pointsData:
                pointCoords.extend([pointData[0].x, pointData[0].y, pointData[0].z])
            existingPointCoords = adsk.fusion.CustomGraphicsCoordinates.create(pointCoords)
            pntSet = graphicsGroup.addPointSet(existingPointCoords, [], adsk.fusion.CustomGraphicsPointTypes.UserDefinedCustomGraphicsPointType, 'Resources/ExistingPoint.png')
            pntSet.depthPriority = 1
    except:
        _ui.messageBox('Unexpected failure:\n{}'.format(traceback.format_exc()))


def CreatePoints(inputs):
    try:
        root = _design.rootComponent
    
        if _isParametric:
            # Get the name of the base feature to create the point in.
            baseFeatureName = _baseFeatureList.selectedItem.name
                        
            if baseFeatureName == 'Create new Base Feature':
                # Create a new base feature.
                baseFeature = root.features.baseFeatures.add()
            else:
                # Get the specified existing base feature.
                baseFeature = root.features.baseFeatures.itemByName(baseFeatureName)
            
            # Add a construction point ot the base feature.
            baseFeature.startEdit()        

            # Create the points.
            for pointData in _pointsData:
                coord = pointData[0]
                name = pointData[1]                
                pointInput = root.constructionPoints.createInput()
                pointInput.setByPoint(coord)
                pointInput.targetBaseOrFormFeature = baseFeature
                point = root.constructionPoints.add(pointInput)
                point.name = name
                
            baseFeature.finishEdit()
                                    
            # Save the name of the base feature as an attribute so it
            # can be used as the default the next time the command is run.
            _design.attributes.add('ekinsPointAtCoord', 'BaseFeatureName', baseFeature.name)
        else:
            # Create the points.
            for pointData in _pointsData:
                coord = pointData[0]
                name = pointData[1]                
                pointInput = root.constructionPoints.createInput()
                pointInput.setByPoint(coord)
                point = root.constructionPoints.add(pointInput)
                point.name = name
    except:
        _ui.messageBox('Unexpected failure:\n{}'.format(traceback.format_exc()))


def run(context):
    try:
        # Get the CommandDefinitions collection.
        cmdDefs = _ui.commandDefinitions
        
        # Create a button command definition.
        buttonDef = cmdDefs.addButtonDefinition('ekinsPointAtCoord', 
                                                'Point at Coordinate', 
                                                'Creates construction point at specified xyz coordinate.',
                                                './Resources/PointAtCoord')
        
        # Connect to the command created event.
        commandCreated = CommandCreatedEventHandler()
        buttonDef.commandCreated.add(commandCreated)
        handlers.append(commandCreated)
        
        # Get the CONSTRUCTION panel. 
        constructionPanel = _ui.allToolbarPanels.itemById('ConstructionPanel')
        
        # Add the button just below the POINT AT VERTEX command.
        buttonControl = constructionPanel.controls.addCommand(buttonDef, 'WorkPointFromPointCommand', False)
    except:
        _ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))


def stop(context):
    try:
        buttonDef = _ui.commandDefinitions.itemById('ekinsPointAtCoord')
        if buttonDef:
            buttonDef.deleteMe()
        
        # Get the CONSTRUCTION panel. 
        constructionPanel = _ui.allToolbarPanels.itemById('ConstructionPanel')
        buttonControl = constructionPanel.controls.itemById('ekinsPointAtCoord')
        if buttonControl:
            buttonControl.deleteMe()       
    except:
        _ui.messageBox('Failed:\n{}'.format(traceback.format_exc()))
